(function (window, undefined) {
	var frm = document.getElementById("frm-generator"),
		domainKeyInpt = frm.elements["domainKey"],
		masterKeyInpt = frm.elements["masterKey"],
		passwordHashInpt = frm.elements["pwHash"],
		pwUnmaskInpt = frm.elements["pwUnmask"],
		nextPwHashBttn = frm.elements["nextPwHash"],
		reqDigitsInpt = frm.elements["reqDigits"],
		reqAlphaInpt = frm.elements["reqAlpha"],
		reqUppercaseInpt = frm.elements["reqUppercase"],
		reqSpecialsInpt = frm.elements["reqSpecials"],
		requiredLengthInpt = frm.elements["reqLength"],
		cnxBttn = frm.elements["btn-cnx"],
		lcgs = [],
		hashPasswordTimer, i;

	function hashPassword() {
		if (!(
		   masterKeyInpt
		   && masterKeyInpt.value
		   && (masterKeyInpt.value.length >= 8)
		   && domainKeyInpt
		   && domainKeyInpt.value
		   && requiredLengthInpt
		   && requiredLengthInpt.value
		   && nextPwHashBttn
		   && nextPwHashBttn.value
		)) {
			passwordHashInpt.value = "";
			return false;
		}

		var characters = {
			digits: "0123456789",
			alpha: "abcdefghijklmnopqrstuvwxyz",
			uppers: "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
			specials: " !\"#$%&'()*+,-./:;<=>?@[\\]^_{|}~"
		},
			chars = "",
			pwHash = "",
			key, keyHash, rnd, charsLen;

		// All: 0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ !"#$%&'()*+,-./:;<=>?@[]^_{|}~
		if (reqDigitsInpt.checked) { chars += characters.digits; }
		if (reqAlphaInpt.checked) { chars += characters.alpha; }
		if (reqUppercaseInpt.checked) { chars += characters.uppers; }
		if (reqSpecialsInpt.checked) { chars += characters.specials; }
		charsLen = chars.length;

		// Create hash message & then generate SHA256 hash
		key = masterKeyInpt.value + domainKeyInpt.value + requiredLengthInpt.value + nextPwHashBttn.value;
		keyHash = Sha256.hash(key); //Ex: 1fc9ad5965454404df0f9f7ee2b7e2f3458cfb68dd48753ab62f7ec1f701c823

		// Create the repeatable random number sequence generators based upon pieces of the key.
		for (i=0; i<64; i+=8) {
			lcgs.push(LinearCongruentialGenerator("0x"+keyHash.substr(i,8)));
		}

		for (i=requiredLengthInpt.value; i>0; --i) {
			rnd = Math.floor(lcgs[i % 8].next() * charsLen);
			if (!chars[rnd]) { console.error(rnd) }
			pwHash += chars[rnd];
		}

		console.log(masterKeyInpt.value, domainKeyInpt.value, requiredLengthInpt.value, nextPwHashBttn.value, chars, charsLen, keyHash, pwHash);

		passwordHashInpt.value = pwHash;
		return true;
	}

	function delayedChange() {
		if (hashPasswordTimer) { clearTimeout(hashPasswordTimer); }
		hashPasswordTimer = window.setTimeout(function () {
			hashPassword();
			hashPasswordTimer = null;
		},400);
	}

	self.on("message", function (frmData) {
		var el;
		for (name in frmData) {
			el = document.getElementById("fld-"+name);
			if (el) {
				el.value = frmData[name];
			}
		}
		hashPassword();
	});

	// Increment the Next Hash button
	nextPwHashBttn.addEventListener("click", function (evt) {
		var noteEl = evt.target.parentNode.querySelector(".note");
		if (!noteEl) {
			noteEl = document.createElement("span");
			noteEl.className = "note";
			evt.target.parentNode.insertBefore(noteEl, evt.target);
		}
		noteEl.innerHTML = "Hash #" + evt.target.value++;
		hashPassword();
	});

	// (un-)Mask the Master Key & Password Hash
	pwUnmaskInpt.addEventListener("click", function (evt) {
		var type = (evt.target.checked) ? "text" : "password";
		passwordHashInpt.type = type;
		masterKeyInpt.type = type;
	});

	// Close the panel on Cancel Button Click
	cnxBttn.addEventListener("click", function (evt) {
		console.log("Generator Panel Hide")
		self.port.emit('hide');
	});

	// Re-Generate the Key each time a requirement or key changes
	domainKeyInpt.addEventListener("keyup", delayedChange);
	masterKeyInpt.addEventListener("keyup", delayedChange);
	reqDigitsInpt.addEventListener("change", hashPassword);
	reqAlphaInpt.addEventListener("change", hashPassword);
	reqUppercaseInpt.addEventListener("change", hashPassword);
	reqSpecialsInpt.addEventListener("change", hashPassword);
	requiredLengthInpt.addEventListener("keyup", delayedChange);

console.log("Generator Panel ContentScript init");

})(this);
